package Tapper::Reports::Web::Model;
our $AUTHORITY = 'cpan:TAPPER';
$Tapper::Reports::Web::Model::VERSION = '5.0.5';
use strict;
use warnings;

use parent 'Catalyst::Model::DBIC::Schema';
use parent 'Tapper::Model';

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Tapper::Reports::Web::Model

=head1 AUTHORS

=over 4

=item *

AMD OSRC Tapper Team <tapper@amd64.org>

=item *

Tapper Team <tapper-ops@amazon.com>

=back

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2016 by Advanced Micro Devices, Inc..

This is free software, licensed under:

  The (two-clause) FreeBSD License

=cut
